/*
 * File         : Session.java
 * Author       : Tomasz Stachowiak
 * Created      : Thu Jun 19 14:54:59 1997
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

import java.sql.*;
import java.util.*;

class Session {

  int id;
  String name;
  String date;
  int active;

  NCSdatabase db;

  Vector parts;
  Vector apps;

  public Session(NCSdatabase database) {
    db = database;

    parts = new Vector();
    apps = new Vector();
  }

  public int getID() {
    
    return id;
  }

  public Vector getParts() {

    return parts;

  }

  public Vector getApps() {

    return apps;

  }

  // UPDATE TRANSACTIONS
  public void start(String name, Participant initiator) {

    String sql;

    getNewID();
    getCurrDate();

    sql = "INSERT INTO session VALUES ("
           +id+",'"+name+"','"+date+", 1)";

    db.updateSQL(sql);

    initiator.join(this);
  }


  public void quit() {

    String sql = "UPDATE session SET active=0 WHERE id = " + id;

    db.updateSQL(sql);
  }

  // QUERY TRANSACTIONS
  public void retrieve(ResultSet rs) throws SQLException {

      id = rs.getInt("id");
      name = rs.getString("name");
      date = rs.getString("curr_date");
      active = rs.getInt("active");

  }

  public void retrieveApps() {

    String sql = "SELECT a.name, a.type, a.platform "
      + "FROM application=a, includes=i WHERE i.ses_id = "
      +id+" AND i.app_name = a.name";

    try {

      ResultSet rs = db.querySQL(sql);

      apps.removeAllElements();

      while(rs.next()) {

	Application app = new Application(db);
	app.retrieve(rs);
	apps.addElement(app);
      }

    } catch (SQLException e) {

      System.out.println("Session: " + e);
    }
  }

  public void retrieveParts() {

    String sql = "SELECT p.id, p.host, p.name, p.first_name, p.type " 
      + "FROM participant=p, participates=ps WHERE ps.ses_id = "
      +id+" AND ps.part_id = p.id AND ps.part_host = p.host ";

    try {

      ResultSet rs = db.querySQL(sql);

      parts.removeAllElements();

      while(rs.next()) {

	Participant part = new Participant(db);
	part.retrieve(rs);
	parts.addElement(part);
      }

    } catch (SQLException e) {

      System.out.println("Session: " + e);
    }
  }


  public String toString() {

    String str = id + " " + name + " " + date;

    if (active == 1) 
      str = str + " active";
    else
      str = str + " not active";

    return str;
  }

  public boolean equals(String str) {

    if (str.compareTo(toString()) == 0)
      return true;
    else
      return false;
  }

  // ADDITIONAL PRIVATE METHODS
  private void getNewID() {

    String sql = "SELECT _seq FROM session";

    try {
      
      ResultSet rs = db.querySQL(sql);
      
      while(rs.next()) {
	id = rs.getInt("_seq");
      }
      
    } catch (SQLException e) {
      
      System.out.println("Session: " + e);
    }

  }

  private void getCurrDate() {

    Calendar cal = Calendar.getInstance();
    date = cal.MONTH+"/"+cal.DATE+"/"+cal.YEAR;
  }

}
